/* filename for temporary file to pass parameters from model m to model m+1 */
filename out 'C:\CotQuestU\out.txt';


/********** SET INPUT FILE, FDR TRESHOLD, AND COT GENERATION CONSTANT ************/

/* Input data file */
data cotdata;
  infile 'C:\CotQuestU\Sample Data\pine.cot';
  input ssDNA Cot;

/* Set FDR threshold, higher values make outlier detection more aggressive 
(0.01 recommended. See H.J.Motulsky and R.E.Brown, BMC Bioinformatics, March 2006) */
%let Q = 0.01;

/* If Cot points were generated using hydroxyapatite (HAP) chromatography then set this variable to 1
   If S1 nuclease digestion was used then set it to 0.44   */
%let cgc = 1;

/***************************************************************************/

	%let alg = marquardt;
	
	
run;


/* run model m=1 with default starting values */
proc nlin data=cotdata method=&alg best=1;      
	bounds f0 f1 k1 >= 0;
	bounds f0 f1 < 1;
	model ssDNA = f0+f1*(1+k1*cot)**(-1*&cgc);
	ODS output parameterestimates=parms1 anova=anova1;
	/* ODS listing parameterestimates=parms1 anova=anova1; */
	OUTPUT out=ssDNAapr1 predicted=ssDNAp;
	parms
		f0=.01 to .26 by .05
		f1=.01 to .96 by .05
		k1=1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2; 
run;
/* obtain upper bound for f0 for model m=2 program */
data f0sum;
	set parms1;
	if parameter = 'f0' or parameter = 'f1';
	keep lowercl;
	run;
proc univariate data=f0sum noprint;
	var lowercl;
	output out=f0sum sum=sum;
	run;
data f0sum;
	set f0sum;
	maxf0=1-sum;
	step = (maxf0 - .01)/5;
	drop sum;
	run;
/* create bounds and step sizes within model m=1 parameter output file for model m=2 program*/
data parms1;
	set parms1;
	lowercl = max(0,lowercl);
	uppercl = min(uppercl,1);
	step = (uppercl-lowercl)/5;
	model = 1;
	run;



/* create model m=2 program */
data _null_;
	file out;
		set parms1 end=last;
		if _n_=1 then do;
			put
				'proc nlin data=cotdata method=&alg best=1;' /
				'bounds f0 f1 k1 f2 k2 >= 0;' /
				'bounds f0 f1 f2 < 1;' /
				'model ssDNA = f0+f1*(1+k1*cot)**(-1*&cgc)+f2*(1+k2*cot)**(-1*&cgc);' /
				'ODS output parameterestimates=parms2 anova=anova2;' /
				'OUTPUT out=ssDNAapr2 predicted=ssDNAp;'
				'parms'
				;
			end;
		if parameter = 'f0' then do;
			put 'f0 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'f1' then do;
			put 'f1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'k1' then do;
			put 'k1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if last then do;
			set f0sum;
				put 'f2 = .01 to ' maxf0 ' by ' step;
			put 'k2 = 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2;' /
			'run;' /
			;
			end;
	run;

/* run model m=2 program */
%include out;


/* obtain upper bound for f0 for model m=3 program */
data f0sum;
	set parms2;
	if parameter = 'f0' or parameter = 'f1' or parameter = 'f2';
	keep lowercl;
	run;
proc univariate data=f0sum noprint;
	var lowercl;
	output out=f0sum sum=sum;
	run;
data f0sum;
	set f0sum;
	maxf0=1-sum;
	step = (maxf0 - .001)/5;
	drop sum;
	run;
/* create bounds and step sizes within model m=2 parameter output file for model m=3 program*/
data parms2;
	set parms2;
	lowercl = max(0,lowercl);
	uppercl = min(uppercl,1);
	step = (uppercl-lowercl)/5;
	model = 2;
	run;

/* create model m=3 program */
data _null_;
	file out;
		set parms2 end=last;
		if _n_=1 then do;
			put
				'proc nlin data=cotdata method=&alg best=1;' /
				'bounds f0 f1 k1 f2 k2 f3 k3 >= 0;' /
				'bounds f0 f1 f2 f3 < 1;' /
				'model ssDNA = f0+f1*(1+k1*cot)**(-1*&cgc)+f2*(1+k2*cot)**(-1*&cgc)+f3*(1+k3*cot)**(-1*&cgc);' /
				'ODS output parameterestimates=parms3 anova=anova3;' /
				'OUTPUT out=ssDNAapr3 predicted=ssDNAp;'
				'parms'
				;
			end;
		if parameter = 'f0' then do;
			put 'f0 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'f1' then do;
			put 'f1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'k1' then do;
			put 'k1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'f2' then do;
			put 'f2 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'k2' then do;
			put 'k2 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if last then do;
			set f0sum;
				put 'f3 = .001 to ' maxf0 ' by ' step;
			put 'k3 = 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2;' /
			'run;' /
			;
			end;
	run;

/* run model m=3 program */
%include out;


/* add residuals */
data ssDNAapr3;
	set ssDNAapr3;
	residual = ssDNA - ssDNAp;
	run;



/* compute sample size N and mean absolute deviation MAD,
	and output to temp dataset work.cauchyparms */
proc univariate data=ssDNAapr3 robustscale noprint;
	var residual;
	output out=cauchyparms1 mad=mad N=n;
	run;


proc sql noprint;

	select Estimate into :f03 from parms3 where Parameter = 'f0';
	select Estimate into :f13 from parms3 where Parameter = 'f1';
	select Estimate into :f23 from parms3 where Parameter = 'f2';
	select Estimate into :f33 from parms3 where Parameter = 'f3';
	select Estimate into :k13 from parms3 where Parameter = 'k1';
	select Estimate into :k23 from parms3 where Parameter = 'k2';
	select Estimate into :k33 from parms3 where Parameter = 'k3';

	select mad into :mad from cauchyparms1;
	select N into :N from cauchyparms1;

	%let halfmad = %sysevalf(&mad/2);
	%let twicemad = %sysevalf(2*&mad);

quit;



/* fit nonlinear regression model with Cauchy (Lorentzian) -distributed error terms,
	with Cauchy scale parameter sc */
proc model data=cotdata noprint;
	dependent ssdna;
	bounds f0 k1 f1 k2 f2 k2 f3 sc >= 0;
	bounds f0 f1 f2 f3 < 1;
	ssdna = f0+f1*(1+k1*cot)**(-1*&cgc)+f2*(1+k2*cot)**(-1*&cgc)+f3*(1+k3*cot)**(-1*&cgc);
	/* fit model and send output to temp file work.cauchyres */
	fit ssdna 
	start=		f0 &f03
				f1 &f13
				k1 &k13
				f2 &f23
				k2 &k23
				f3 &f33
				k3 &k33
				sc &halfmad
				sc &mad 
				sc &twicemad
			/ out = cauchyres outall
		;
	/* likelihood function for the residuals, based on scale-Cauchy family */ 
	obj = log(3.1415926*sc*(1+(resid.ssdna/sc)**2));
	errormodel ssdna ~ general(obj);
	run;


/* create dataset to compute rsdr */
data rsdrdata;
	set cauchyres (rename=(ssdna=residual));
	if _TYPE_='RESIDUAL';
	cabsres = abs(residual);
	run;
/* compute rsdr from cauchy residuals */
proc univariate data=rsdrdata noprint;
	var cabsres;
	output out=cauchyparms2 pctlpre=q_ PCTLPTS=68.27;
	run;

proc sql noprint;
	select * into :qntl from cauchyparms2;
quit;

/* create dataset of actual (observed) values */
data cauchyres1;
	set cauchyres (rename=(ssdna=actual));
	if _TYPE_='ACTUAL';
	run;
/* create dataset of predicted (fitted) values from Cauchy model */
data cauchyres2;
	set cauchyres (rename=(ssdna=cpredict));
	if _TYPE_='PREDICT';
	run;
/* create master dataset for PROC MULTTEST */
data cauchyres;
	merge cauchyres1 cauchyres2;
	ctrueres = actual-cpredict;
	cnormres = ctrueres/(&qntl*&N/(&N-7));
	raw_p = 2*(1-CDF('T',abs(cnormres),(&N-7))); 
	keep cot actual cpredict ctrueres cnormres raw_p;
	run;
/* run proc multtest to obtain fdr-corrected p-values */
proc multtest pdata=cauchyres fdr out=cauchyres noprint;
	run;
/* revise temp dataset to flag nominated outliers
		based on fdr-corrected p-values with threshold = .01 */
data cauchyres;
	set cauchyres;
	outlier = 0;
	if fdr_p<&Q then outlier=1;
	run;


/* Create output report */
ods html path='c:\CotQuestU\OutlierDetection' file='Outliers.htm'; 
proc print data=cauchyres;
format actual cot 8.7;
run;
ods html close;

