/* filename for temporary file to pass parameters from model m to model m+1 */
filename out 'C:\CotQuestU\out.txt';

/**** SET INPUT FILE, ORGANISM, FITTING ALGORITHM, PERMISSIBLE OVERLAP, AND COT GENERATION CONST ***/
data cotdata;

/* Input data file with path */
%let file = C:\CotQuestU\Sample Data\ecoli.cot;
  infile "'&file'";
  input ssDNA Cot;

/* Enter organism name (it will be included in the output report) */
%let organism = Escherichia coli;

/* Select fitting algorithm. 
Options: 
gauss (recommended) 
marquardt (recommended when parameter correlation is high, > 0.8 or < -0.8)
newton (not recommneded)*/
%let alg = gauss;

/* Select permissible percentage of fraction overlap (in log10 scale)
 Fraction overlaps exceeding this value will be shown in red in the Cot Analysis Summary section 
 of the report. Incorrect models may have high overlap values. Recommeded value is 0.5 (50%). */
%let permis_fraction_overlap = 0.5; 

/* If Cot points were generated using hydroxyapatite (HAP) chromatography then set this variable to 1
   If S1 nuclease digestion was used then set it to 0.44   */
%let cgc = 1;


/***************************************************************************/

run;



proc sql noprint;
	select count(*) into :n_obs from Cotdata;
quit;

/* run model m=1 with default starting values */
proc nlin data=cotdata method=&alg best=1;      
	bounds f0 f1 k1 >= 0;
	bounds f0 f1 < 1;
	model ssDNA = f0+f1*(1+k1*cot)**(-1*&cgc);
	ODS output convergencestatus=conv1 parameterestimates=parms1 anova=anova1;
	/* ODS listing parameterestimates=parms1 anova=anova1; */
	OUTPUT out=ssDNAapr1 predicted=ssDNAp;
	parms
		f0=.01 to .26 by .05
		f1=.01 to .96 by .05
		k1=1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2; 
run;
/* obtain upper bound for f0 for model m=2 program */
data f0sum;
	set parms1;
	if parameter = 'f0' or parameter = 'f1';
	keep lowercl;
	run;
proc univariate data=f0sum noprint;
	var lowercl;
	output out=f0sum sum=sum;
	run;
data f0sum;
	set f0sum;
	maxf0=1-sum;
	step = (maxf0 - .01)/5;
	drop sum;
	run;
/* create bounds and step sizes within model m=1 parameter output file for model m=2 program*/
data parms1;
	set parms1;
	lowercl = max(0,lowercl);
	uppercl = min(uppercl,1);
	step = (uppercl-lowercl)/5;
	model = 1;
	run;

/* add residuals */
data ssDNAapr1;
	set ssDNAapr1;
	residual = ssDNA - ssDNAp;
	observation = _N_;
	run;

goptions reset=all device=gif gsfname=gout;
title 'Normal Probability Plot of the Residuals (m=1)';
filename gout "C:\CotQuestU\ReportFiles\m1_qqPlot.gif"; 
proc univariate data=ssDNAapr1 normal noprint;	
	var residual;
 	qqplot residual / normal(mu=est sigma=est) pctlscale;
run;

goptions reset=all device=gif gsfname=gout;
title 'Histogram of the Residuals (m=1)';
filename gout "C:\CotQuestU\ReportFiles\m1_histogram.gif"; 
proc univariate data=ssDNAapr1 normal noprint;
	var residual;
 	histogram residual / cfill=ltgray normal(color=red MU=0) kernel(color=blue);
	inset normal(ad adpval cvm cvmpval) / cfill  = ywh format = 7.6;
	ODS output GoodnessOfFit=univ1;
run;

goptions reset=all device=gif gsfname=gout;
title 'Residuals Versus the Fitted Values (m=1)';
filename gout "C:\CotQuestU\ReportFiles\m1_Res_ssDNAp.gif"; 
proc gplot data=ssDNAapr1;
	plot residual*ssDNAp /vref=0;	
	symbol1 v=dot c=blue i=none w=0.5 h=0.5;
run;

goptions reset=all device=gif gsfname=gout;
title 'Residuals Versus the Order of the Data (m=1)';
filename gout "C:\CotQuestU\ReportFiles\m1_Res_Obs.gif"; 
proc gplot data=ssDNAapr1;
	plot residual*observation /vref=0;
	symbol1 v=dot c=red ci=blue i=join w=0.5 h=0.5;
run;
title;

/* create model m=2 program */
data _null_;
	file out;
		set parms1 end=last;
		if _n_=1 then do;
			put
				'proc nlin data=cotdata method=&alg best=1;' /
				'bounds f0 f1 k1 f2 k2 >= 0;' /
				'bounds f0 f1 f2 < 1;' /
				'model ssDNA = f0+f1*(1+k1*cot)**(-1*&cgc)+f2*(1+k2*cot)**(-1*&cgc);' /
				'ODS output convergencestatus=conv2 parameterestimates=parms2 anova=anova2;' /
				'OUTPUT out=ssDNAapr2 predicted=ssDNAp;'
				'parms'
				;
			end;
		if parameter = 'f0' then do;
			put 'f0 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'f1' then do;
			put 'f1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'k1' then do;
			put 'k1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if last then do;
			set f0sum;
				put 'f2 = .01 to ' maxf0 ' by ' step;
			put 'k2 = 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2;' /
			'run;' /
			;
			end;
	run;

/* run model m=2 program */
ods html path = "C:\CotQuestU\ReportFiles" (url=none) file='NLIN2.htm';
%include out;
ods html close;

/* find the lowest k for model 2 */
proc sql noprint;
	select Parameter into :lk2 from Parms2 where Estimate in (select Min(Estimate) from Parms2 where Parameter eqt 'k') and Parameter eqt 'k';
	%let lk2 = %sysfunc(compress(&lk2));
quit;




%macro par2;
%if &lk2 = k2 %then 
%do;
		if parameter = 'k1' then do;
			put 'k1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if last then do;
			set f0sum;
				put 'f2 = .01 to ' maxf0 ' by ' step;
			put ';' /
%end;
%else
%do;
		if last then do;
			set f0sum;
				put 'f2 = .01 to ' maxf0 ' by ' step;
			put 'k2 = 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2;' /
%end;

%mend;


/* obtain upper bound for f0 for model m=3 program */
data f0sum;
	set parms2;
	if parameter = 'f0' or parameter = 'f1' or parameter = 'f2';
	keep lowercl;
	run;
proc univariate data=f0sum noprint;
	var lowercl;
	output out=f0sum sum=sum;
	run;
data f0sum;
	set f0sum;
	maxf0=1-sum;
	step = (maxf0 - .001)/5;
	drop sum;
	run;
/* create bounds and step sizes within model m=2 parameter output file for model m=3 program*/
data parms2;
	set parms2;
	lowercl = max(0,lowercl);
	uppercl = min(uppercl,1);
	step = (uppercl-lowercl)/5;
	model = 2;
	run;


/* add residuals */
data ssDNAapr2;
	set ssDNAapr2;
	residual = ssDNA - ssDNAp;
	observation = _N_;
	run;



goptions reset=all device=gif gsfname=gout;
title 'Normal Probability Plot of the Residuals (m=2)';
filename gout "C:\CotQuestU\ReportFiles\m2_qqPlot.gif"; 
proc univariate data=ssDNAapr2 normal noprint;	
	var residual;
 	qqplot residual / normal(mu=est sigma=est) pctlscale;
run;



goptions reset=all device=gif gsfname=gout;
title 'Histogram of the Residuals (m=2)';
filename gout "C:\CotQuestU\ReportFiles\m2_histogram.gif"; 
proc univariate data=ssDNAapr2 normal noprint;	
	var residual;
 	histogram residual / cfill=ltgray normal(color=red MU=0) kernel(color=blue);
	inset normal(ad adpval cvm cvmpval) / cfill  = ywh format = 7.6;
	ODS output GoodnessOfFit=univ2;
run;


goptions reset=all device=gif gsfname=gout;
title 'Residuals Versus the Fitted Values (m=2)';
filename gout "C:\CotQuestU\ReportFiles\m2_Res_ssDNAp.gif";
proc gplot data=ssDNAapr2;
	plot residual*ssDNAp /vref=0;	
	symbol1 v=dot c=blue i=none w=0.5 h=0.5;
run;


goptions reset=all device=gif gsfname=gout;
title 'Residuals Versus the Order of the Data (m=2)';
filename gout "C:\CotQuestU\ReportFiles\m2_Res_Obs.gif";
proc gplot data=ssDNAapr2;
	plot residual*observation /vref=0;
	symbol1 v=dot c=red ci=blue i=join w=0.5 h=0.5;
run;

title;


/* create model m=3 program */
data _null_;
	file out;
		set parms2 end=last;
		if _n_=1 then do;
			put
				'proc nlin data=cotdata method=&alg best=1;' /
				'bounds f0 f1 k1 f2 k2 f3 k3 >= 0;' /
				'bounds f0 f1 f2 f3 < 1;' /
				'model ssDNA = f0+f1*(1+k1*cot)**(-1*&cgc)+f2*(1+k2*cot)**(-1*&cgc)+f3*(1+k3*cot)**(-1*&cgc);' /
				'ODS output convergencestatus=conv3 parameterestimates=parms3 anova=anova3;' /
				'OUTPUT out=ssDNAapr3 predicted=ssDNAp;'
				'parms'
				;
			end;
		if parameter = 'f0' then do;
			put 'f0 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'f1' then do;
			put 'f1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'k1' then do;
			put 'k1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'f2' then do;
			put 'f2 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'k2' then do;
			put 'k2 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if last then do;
			set f0sum;
				put 'f3 = .001 to ' maxf0 ' by ' step;
			put 'k3 = 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2;' /
			'run;' /
			;
			end;
	run;

/* run model m=3 program */
ods html path = "C:\CotQuestU\ReportFiles" (url=none) file='NLIN3.htm'; 
%include out;
ods html close;

/* find the lowest k for model 3 */
proc sql noprint;
	select Parameter into :lk3 from Parms3 where Estimate in (select Min(Estimate) from Parms3 where Parameter eqt 'k') and Parameter eqt 'k';
	%let lk3 = %sysfunc(compress(&lk3));
quit;



/* obtain upper bound for f0 for model m=4 program */
data f0sum;
	set parms3;
	if parameter = 'f0' or parameter = 'f1' or parameter = 'f2' or parameter = 'f3';
	keep lowercl;
	run;
proc univariate data=f0sum noprint;
	var lowercl;
	output out=f0sum sum=sum;
	run;
data f0sum;
	set f0sum;
	maxf0=1-sum;
	step = (maxf0 - .0001)/5;
	drop sum;
	run;
/* create bounds and step sizes within model m=3 parameter output file for model m=4 program*/
data parms3;
	set parms3;
	lowercl = max(0,lowercl);
	uppercl = min(uppercl,1);
	step = (uppercl-lowercl)/5;
	model = 3;
	run;


/* add residuals */
data ssDNAapr3;
	set ssDNAapr3;
	residual = ssDNA - ssDNAp;
	observation = _N_;
	run;



goptions reset=all device=gif gsfname=gout;
title 'Normal Probability Plot of the Residuals (m=3)';
filename gout "C:\CotQuestU\ReportFiles\m3_qqPlot.gif"; 
proc univariate data=ssDNAapr3 normal noprint;
	var residual;
 	qqplot residual / normal(mu=est sigma=est) pctlscale;
run;


goptions reset=all device=gif gsfname=gout;
title 'Histogram of the Residuals (m=3)';
filename gout "C:\CotQuestU\ReportFiles\m3_histogram.gif"; 
proc univariate data=ssDNAapr3 normal noprint;
	var residual;
 	histogram residual / cfill=ltgray normal(color=red MU=0) kernel(color=blue);
	inset normal(ad adpval cvm cvmpval) / cfill  = ywh format = 7.6;
	ODS output GoodnessOfFit=univ3;
run;


goptions reset=all device=gif gsfname=gout;
title 'Residuals Versus the Fitted Values (m=3)';
filename gout "C:\CotQuestU\ReportFiles\m3_Res_ssDNAp.gif";
proc gplot data=ssDNAapr3;	
	plot residual*ssDNAp /vref=0;	
	symbol1 v=dot c=blue i=none w=0.5 h=0.5;
run;


goptions reset=all device=gif gsfname=gout;
title 'Residuals Versus the Order of the Data (m=3)';
filename gout "C:\CotQuestU\ReportFiles\m3_Res_Obs.gif"; 
proc gplot data=ssDNAapr3;
	plot residual*observation /vref=0;
	symbol1 v=dot c=red ci=blue i=join w=0.5 h=0.5;
run;

title;

/* create model m=4 program */
data _null_;
	file out;
		set parms3 end=last;
		if _n_=1 then do;
			put
				'proc nlin data=cotdata method=&alg best=1;' /
				'bounds f0 f1 k1 f2 k2 f3 k3 f4 k4 >= 0;' /
				'bounds f0 f1 f2 f3 f4 < 1;' /
				'model ssDNA = f0+f1*(1+k1*cot)**(-1*&cgc)+f2*(1+k2*cot)**(-1*&cgc)+f3*(1+k3*cot)**(-1*&cgc)+f4*(1+k4*cot)**(-1*&cgc);' /
				'ODS output convergencestatus=conv4 parameterestimates=parms4 anova=anova4;' /
				'OUTPUT out=ssDNAapr4 predicted=ssDNAp;' 
				'parms'
				;
			end;
		if parameter = 'f0' then do;
			put 'f0 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'f1' then do;
			put 'f1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'k1' then do;
			put 'k1 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'f2' then do;
			put 'f2 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'k2' then do;
			put 'k2 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'f3' then do;
			put 'f3 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if parameter = 'k3' then do;
			put 'k3 =' lowercl ' to ' uppercl ' by ' step;
		end;
		if last then do;
			set f0sum;
				put 'f4 = .0001 to ' maxf0 ' by ' step;
			put 'k4 = 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2;' /
			'run;' /
			;
			end;
	run;

/* run model m=4 program */
ods html path = "C:\CotQuestU\ReportFiles" (url=none) file='NLIN4.htm'; 
%include out;
ods html close;

/* find the lowest k for model 4 */
proc sql noprint;
	select Parameter into :lk4 from Parms4 where Estimate in (select Min(Estimate) from Parms4 where Parameter eqt 'k') and Parameter eqt 'k';
	%let lk4 = %sysfunc(compress(&lk4));
quit;



data parms4;
	set parms4;
	lowercl = max(0,lowercl);
	uppercl = min(uppercl,1);
	step = (uppercl-lowercl)/5;
	model = 4;
	run;


/* add residuals */
data ssDNAapr4;
	set ssDNAapr4;
	residual = ssDNA - ssDNAp;
	observation = _N_;
	run;



goptions reset=all device=gif gsfname=gout;
title 'Normal Probability Plot of the Residuals (m=4)';
filename gout "C:\CotQuestU\ReportFiles\m4_qqPlot.gif"; 
proc univariate data=ssDNAapr4 normal noprint; 
	var residual;
 	qqplot residual / normal(mu=est sigma=est) pctlscale;
run;

goptions reset=all device=gif gsfname=gout;
title 'Histogram of the Residuals (m=4)';
filename gout "C:\CotQuestU\ReportFiles\m4_histogram.gif"; 
proc univariate data=ssDNAapr4 normal noprint;
 	histogram residual / cfill=ltgray normal(color=red MU=0) kernel(color=blue);
	inset normal(ad adpval cvm cvmpval) / cfill  = ywh format = 7.6;
	ODS output GoodnessOfFit=univ4;
run;


goptions reset=all device=gif gsfname=gout;
title 'Residuals Versus the Fitted Values (m=4)';
filename gout "C:\CotQuestU\ReportFiles\m4_Res_ssDNAp.gif"; 
proc gplot data=ssDNAapr4;	
	plot residual*ssDNAp /vref=0;	
	symbol1 v=dot c=blue i=none w=0.5 h=0.5;
run;


goptions reset=all device=gif gsfname=gout;
title 'Residuals Versus the Order of the Data (m=4)';
filename gout "C:\CotQuestU\ReportFiles\m4_Res_Obs.gif"; 
proc gplot data=ssDNAapr4;
	plot residual*observation /vref=0;
	symbol1 v=dot c=red ci=blue i=join w=0.5 h=0.5;
run;



/* cancel subsequent title printing */
title;

/* create model parameter estimate comparison output dataset */
data parmsfinal;
	set
		parms1
		parms2
		parms3
		parms4
		;
	drop step;
run;


/* create model fit comparison output dataset including AIC */
data anova1;
	set anova1;
	model = 1;
	if source = 'Error' then do;
		AIC = &n_obs*log(SS/&n_obs)+2*(&n_obs-DF+1);
		AICc = AIC + 2*(&n_obs-DF+1)*((&n_obs-DF+1)+1)/(&n_obs-(&n_obs-DF+1)-1);
		end;
	run;
data anova2;
	set anova2;
	model = 2;
	if source = 'Error' then do;
		AIC = &n_obs*log(SS/&n_obs)+2*(&n_obs-DF+1);
		AICc = AIC + 2*(&n_obs-DF+1)*((&n_obs-DF+1)+1)/(&n_obs-(&n_obs-DF+1)-1);
		end;
	run;
data anova3;
	set anova3;
	model = 3;
	if source = 'Error' then do;
		AIC = &n_obs*log(SS/&n_obs)+2*(&n_obs-DF+1);
		AICc = AIC + 2*(&n_obs-DF+1)*((&n_obs-DF+1)+1)/(&n_obs-(&n_obs-DF+1)-1);
		end;
	run;
data anova4;
	set anova4;
	model = 4;
	if source = 'Error' then do;
		AIC = &n_obs*log(SS/&n_obs)+2*(&n_obs-DF+1);
		AICc = AIC + 2*(&n_obs-DF+1)*((&n_obs-DF+1)+1)/(&n_obs-(&n_obs-DF+1)-1);
		end;
	run;
data anovafinal;
	set
		anova1
		anova2
		anova3
		anova4
		;
	run;





/* print summary results */
ods html path = "C:\CotQuestU\ReportFiles" (url=none) file='Parameters.htm'; 

/* ods html close;

ods html file="C:\CotQuestU\ReportFiles\Parameters.htm"; */
proc print data=parmsfinal;
run;
ods html close;

ods html file="C:\CotQuestU\ReportFiles\Statistics.htm";
proc print data=anovafinal;
run;
ods html close;


ods html file="C:\CotQuestU\ReportFiles\Res1.htm";
proc print data=ssDNAapr1;  
run;
ods html close;


ods html file="C:\CotQuestU\ReportFiles\Res2.htm";
proc print data=ssDNAapr2;  
run;
ods html close;

ods html file="C:\CotQuestU\ReportFiles\Res3.htm";
proc print data=ssDNAapr3;  
run;
ods html close;


ods html file="C:\CotQuestU\ReportFiles\Res4.htm";
proc print data=ssDNAapr4;  
run;
ods html close;




%macro bm;
	/*%if %sysevalf(&f01<=0) or %sysevalf(&f11<=0) or %sysevalf(&k11<=0) %then %let bm1 = 1;*/
	%if %sysevalf(&f02<=0) or %sysevalf(&f12<=0) or %sysevalf(&f22<=0) or %sysevalf(&k12<=0) or %sysevalf(&k22<=0) %then %let bm2 = 2; 
	%if %sysevalf(&f03<=0) or %sysevalf(&f13<=0) or %sysevalf(&f23<=0) or %sysevalf(&f33<=0) or %sysevalf(&k13<=0) or %sysevalf(&k23<=0) or %sysevalf(&k33<=0) %then %let bm3 = 3;
	%if %sysevalf(&f04<=0) or %sysevalf(&f14<=0) or %sysevalf(&f24<=0) or %sysevalf(&f34<=0) or %sysevalf(&f44<=0) or %sysevalf(&k14<=0) or %sysevalf(&k24<=0) or %sysevalf(&k34<=0) or %sysevalf(&k44<=0) %then %let bm4 = 4;

	/*%if %sysevalf(&f01>=1) or %sysevalf(&f11>=1) %then %let bm1 = 1;*/
	%if %sysevalf(&f02>=1) or %sysevalf(&f12>=1) or %sysevalf(&f22>=1) %then %let bm2 = 2; 
	%if %sysevalf(&f03>=1) or %sysevalf(&f13>=1) or %sysevalf(&f23>=1) or %sysevalf(&f33>=1) %then %let bm3 = 3;
	%if %sysevalf(&f04>=1) or %sysevalf(&f14>=1) or %sysevalf(&f24>=1) or %sysevalf(&f34>=1) or %sysevalf(&f44>=1) %then %let bm4 = 4;

%mend;

proc sql noprint;

	%let bm1 = 0;
	%let bm2 = 0;
	%let bm3 = 0;
	%let bm4 = 0;

	%let fbm1 = 0;
	%let fbm2 = 0;
	%let fbm3 = 0;
	%let fbm4 = 0;
	%global fk12 fk22 fk13 fk23 fk33 fk14 fk24 fk34 fk44;

	select Estimate into :f01 from parmsfinal where model = 1 and Parameter = 'f0';
	select Estimate into :f11 from parmsfinal where model = 1 and Parameter = 'f1';
	select Estimate into :k11 from parmsfinal where model = 1 and Parameter = 'k1';
	
	select Estimate into :f02 from parmsfinal where model = 2 and Parameter = 'f0';
	select Estimate into :f12 from parmsfinal where model = 2 and Parameter = 'f1';
	select Estimate into :f22 from parmsfinal where model = 2 and Parameter = 'f2';
	select Estimate into :k12 from parmsfinal where model = 2 and Parameter = 'k1';
	select Estimate into :k22 from parmsfinal where model = 2 and Parameter = 'k2';

	select Estimate into :f03 from parmsfinal where model = 3 and Parameter = 'f0';
	select Estimate into :f13 from parmsfinal where model = 3 and Parameter = 'f1';
	select Estimate into :f23 from parmsfinal where model = 3 and Parameter = 'f2';
	select Estimate into :f33 from parmsfinal where model = 3 and Parameter = 'f3';
	select Estimate into :k13 from parmsfinal where model = 3 and Parameter = 'k1';
	select Estimate into :k23 from parmsfinal where model = 3 and Parameter = 'k2';
	select Estimate into :k33 from parmsfinal where model = 3 and Parameter = 'k3';

	select Estimate into :f04 from parmsfinal where model = 4 and Parameter = 'f0';
	select Estimate into :f14 from parmsfinal where model = 4 and Parameter = 'f1';
	select Estimate into :f24 from parmsfinal where model = 4 and Parameter = 'f2';
	select Estimate into :f34 from parmsfinal where model = 4 and Parameter = 'f3';
	select Estimate into :f44 from parmsfinal where model = 4 and Parameter = 'f4';
	select Estimate into :k14 from parmsfinal where model = 4 and Parameter = 'k1';
	select Estimate into :k24 from parmsfinal where model = 4 and Parameter = 'k2';
	select Estimate into :k34 from parmsfinal where model = 4 and Parameter = 'k3';
	select Estimate into :k44 from parmsfinal where model = 4 and Parameter = 'k4';

	%bm

	select max(Cot) into :ubn from Cotdata;
	select min(Cot) into :lbn from Cotdata;
	
	/* Find the lowest AICc model without 0-parameters */
	select model into :m from anovafinal where AICc in (select min(AICc) from anovafinal where Source = 'Error' and model <> &bm1 and model <> &bm2 and model <> &bm3 and model <> &bm4);
	
	/* Create variables for the summary */
	drop table Parameters;
	create table Parameters (id num, k num, f num, model num);
	insert into Parameters set id = 1, k = &k12, f = &f12, model = 2;
	insert into Parameters set id = 2, k = &k22, f = &f22, model = 2;

	insert into Parameters set id = 1, k = &k13, f = &f13, model = 3;
	insert into Parameters set id = 2, k = &k23, f = &f23, model = 3;
	insert into Parameters set id = 3, k = &k33, f = &f33, model = 3;

	insert into Parameters set id = 1, k = &k14, f = &f14, model = 4;
	insert into Parameters set id = 2, k = &k24, f = &f24, model = 4;
	insert into Parameters set id = 3, k = &k34, f = &f34, model = 4;
	insert into Parameters set id = 4, k = &k44, f = &f44, model = 4;

	select id, k, f into :id_sl2, :k_sl2, :f_sl2 from Parameters where model=2 and k in (select min(k) from Parameters where model=2); 
	select k, f into :k_hr2, :f_hr2 from Parameters where model=2 and id<>&id_sl2; 

	select id, k, f into :id_sl3, :k_sl3, :f_sl3 from Parameters where model=3 and k in (select min(k) from Parameters where model=3); 
	select id, k, f into :id_mr3, :k_mr3, :f_mr3 from Parameters where model=3 and k in (select min(k) from Parameters where model=3 and k>=&k_sl3 and id<>&id_sl3) and id<>&id_sl3; 
	select k, f into :k_hr3, :f_hr3 from Parameters where model=3 and id<>&id_sl3 and id<>&id_mr3; 

	select id, k, f into :id_sl4, :k_sl4, :f_sl4 from Parameters where model=4 and k in (select min(k) from Parameters where model=4); 
	select id, k, f into :id_mr4, :k_mr4, :f_mr4 from Parameters where model=4 and k in (select min(k) from Parameters where model=4 and k>=&k_sl4 and id<>&id_sl4) and id<>&id_sl4;
	select id, k, f into :id_hr4, :k_hr4, :f_hr4 from Parameters where model=4 and k in (select min(k) from Parameters where model=4 and k>=&k_mr4 and id<>&id_mr4 and id<>&id_sl4) and id<>&id_mr4 and id<>&id_sl4; 
	select k, f into :k_vhr4, :f_vhr4 from Parameters where model=4 and id<>&id_sl4 and id<>&id_mr4 and id<>&id_hr4;


	
	select AICc into :AICc1 from anovafinal where Source = 'Error' and model = 1;
	select AICc into :AICc2 from anovafinal where Source = 'Error' and model = 2;
	select AICc into :AICc3 from anovafinal where Source = 'Error' and model = 3;
	select AICc into :AICc4 from anovafinal where Source = 'Error' and model = 4;

	select Stat into :Asq1 from univ1 where  TestLab = 'A-Sq';
	select Stat into :Asq2 from univ2 where  TestLab = 'A-Sq'; 
	select Stat into :Asq3 from univ3 where  TestLab = 'A-Sq';
	select Stat into :Asq4 from univ4 where  TestLab = 'A-Sq';
	
	select Stat into :Wsq1 from univ1 where  TestLab = 'W-Sq';
	select Stat into :Wsq2 from univ2 where  TestLab = 'W-Sq'; 
	select Stat into :Wsq3 from univ3 where  TestLab = 'W-Sq';
	select Stat into :Wsq4 from univ4 where  TestLab = 'W-Sq';
	
quit;

%macro status;
%if &Status = 0 %then %let Status = Converged;
%else %if &Status = 1 %then %let Status = <font color=red>Partially Converged</font>;
%else %if &Status = 3 %then %let Status = <font color=red>Failed to Converge</font>;
%mend;

proc sql noprint;

	select status into :Status from conv1;
	%status
	%let StatusConv1 = &Status;
 
	select status into :Status from conv2;
	%status
	%let StatusConv2 = &Status;


	select status into :Status from conv3;
	%status
	%let StatusConv3 = &Status;

	
	select status into :Status from conv4;
	%status
	%let StatusConv4 = &Status;

	

quit;


%macro equations;

%if &bm1 = 0 %then %do; 
	ssDNAp1 = &f01+&f11*(1+&k11*cot)**(-1*&cgc); 
%end; 

%if &bm2 = 0 %then %do; 
	ssDNAp2 = &f02+&f12*(1+&k12*cot)**(-1*&cgc)+&f22*(1+&k22*cot)**(-1*&cgc);
%end;

%if &bm3 = 0 %then %do;
	ssDNAp3 = &f03+&f13*(1+&k13*cot)**(-1*&cgc)+&f23*(1+&k23*cot)**(-1*&cgc)+&f33*(1+&k33*cot)**(-1*&cgc);
%end;

%if &bm4 = 0 %then	%do;
	ssDNAp4 = &f04+&f14*(1+&k14*cot)**(-1*&cgc)+&f24*(1+&k24*cot)**(-1*&cgc)+&f34*(1+&k34*cot)**(-1*&cgc)+&f44*(1+&k44*cot)**(-1*&cgc); 
%end;


%mend;

/*Calculate 3001 points for a high-precision graph extended half order of magnitude on both ends */
data cotscan; 

  do iterations = log10(&lbn/5) to log10(&ubn*5) by (log10(&ubn*5)-log10(&lbn/5))/3000; 
	cot = 10**(iterations);
	
	%equations
	output;  
  end; 

  drop iterations;
run;


/* Merge with the existing dataset */
data cotdata2; 
  set cotdata cotscan; 
run;

proc sort data=cotdata2;
   by cot;
run;

ods html file="C:\CotQuestU\ReportFiles\GraphPoints.htm";
proc print data=cotdata2;
format _all_ 7.6;
run; 
ods html close;


/* Model 1 graph */

%macro m1plot;

title 'Model 1';
%let k121 = %sysevalf(0.1/&k12);
%let k122 = %sysevalf(1/&k12);
%let k123 = %sysevalf(10/&k12);

plot ssDNA*cot ssDNAp1*cot 
	/haxis=axis1 vaxis=axis2
	href=(&k121 &k122 &k123) 
	chref=(blue blue blue) lhref=(46 1 46)
	overlay;

%mend;
goptions reset=all device=gif gsfname=gout;
filename gout "C:\CotQuestU\ReportFiles\CotGraph1.gif"; 

proc gplot data=cotdata2;

axis1 logbase=10 logstyle=expand; 
axis2 minor=none;
%m1plot

symbol1 v=dot c=black i=none w=0.2 h=0.2;
symbol2 c=red v=dot w=0.1 h=0.1 i=spline line=1;

run;



/* Model 2 graph */

%macro m2plot;
%if &bm2=0 %then %do;
title 'Model 2';
%let k121 = %sysevalf(0.1/&k12);
%let k122 = %sysevalf(1/&k12);
%let k123 = %sysevalf(10/&k12);
%let k221 = %sysevalf(0.1/&k22);
%let k222 = %sysevalf(1/&k22);
%let k223 = %sysevalf(10/&k22);


plot ssDNA*cot ssDNAp2*cot 
	/haxis=axis1 vaxis=axis2
	href=(&k121 &k122 &k123 &k221 &k222 &k223) 
	chref=(blue blue blue green green green) lhref=(46 1 46 46 1 46)
	overlay;
%end;
%else %do;
title 'Model 2 is invalid, at least one of its parameters is out of bounds.';
plot ssDNA*cot /haxis=axis1 vaxis=axis2;
%end;

%mend;
goptions reset=all device=gif gsfname=gout;
filename gout "C:\CotQuestU\ReportFiles\CotGraph2.gif"; 

proc gplot data=cotdata2;

axis1 logbase=10 logstyle=expand; 
axis2 minor=none;
%m2plot

symbol1 v=dot c=black i=none w=0.2 h=0.2;
symbol2 c=red v=dot w=0.1 h=0.1 i=spline line=1;

run;


/* Model 3 graph */

%macro m3plot;
%if &bm3=0 %then %do;
title 'Model 3';
%let k131 = %sysevalf(0.1/&k13);
%let k132 = %sysevalf(1/&k13);
%let k133 = %sysevalf(10/&k13);
%let k231 = %sysevalf(0.1/&k23);
%let k232 = %sysevalf(1/&k23);
%let k233 = %sysevalf(10/&k23);
%let k331 = %sysevalf(0.1/&k33);
%let k332 = %sysevalf(1/&k33);
%let k333 = %sysevalf(10/&k33);

plot ssDNA*cot ssDNAp3*cot 
	/haxis=axis1 vaxis=axis2
	href=(&k131 &k132 &k133 &k231 &k232 &k233 &k331 &k332 &k333) 
	chref=(blue blue blue green green green red red red) lhref=(46 1 46 46 1 46 46 1 46)
	overlay;
%end;
%else %do;
title 'Model 3 is invalid, at least one of its parameters is out of bounds.';
plot ssDNA*cot /haxis=axis1 vaxis=axis2;
%end;
%mend;

goptions reset=all device=gif gsfname=gout;
filename gout "C:\CotQuestU\ReportFiles\CotGraph3.gif"; 

proc gplot data=cotdata2;

axis1 logbase=10 logstyle=expand; 
axis2 minor=none;
%m3plot
symbol1 v=dot c=black i=none w=0.2 h=0.2;
symbol2 c=red v=dot w=0.1 h=0.1 i=spline line=1;

run;

/* Model 4 graph */

%macro m4plot;
%if &bm4 = 0 %then %do;
title 'Model 4';
%let k141 = %sysevalf(0.1/&k14);
%let k142 = %sysevalf(1/&k14);
%let k143 = %sysevalf(10/&k14);
%let k241 = %sysevalf(0.1/&k24);
%let k242 = %sysevalf(1/&k24);
%let k243 = %sysevalf(10/&k24);
%let k341 = %sysevalf(0.1/&k34);
%let k342 = %sysevalf(1/&k34);
%let k343 = %sysevalf(10/&k34);
%let k441 = %sysevalf(0.1/&k44);
%let k442 = %sysevalf(1/&k44);
%let k443 = %sysevalf(10/&k44);

plot ssDNA*cot ssDNAp4*cot 
	/haxis=axis1 vaxis=axis2 
	href=(&k141 &k142 &k143 &k241 &k242 &k243 &k341 &k342 &k343 &k441 &k442 &k443) 
	chref=(blue blue blue green green green red red red gray gray gray) lhref=(46 1 46 46 1 46 46 1 46 46 1 46)
	overlay;
%end;
%else %do;
title 'Model 4 is invalid, at least one of its parameters is out of bounds.';
plot ssDNA*cot /haxis=axis1 vaxis=axis2;
%end;
%mend;

goptions reset=all device=gif gsfname=gout;
filename gout "C:\CotQuestU\ReportFiles\CotGraph4.gif"; 

proc gplot data=cotdata2;

axis1 logbase=10 logstyle=expand; 
axis2 minor=none;

%m4plot


symbol1 v=dot c=black i=none w=0.2 h=0.2;
symbol2 c=red v=dot w=0.1 h=0.1 i=spline line=1;

run;


title;


%macro sym(m);

%if &m = 1 %then %do;
symbol2 c=red v=dot w=0.2 h=0.2 i=spline line=1;
symbol3 c=green v=dot w=0.05 h=0.05 i=spline line=1;
symbol4 c=blue v=dot w=0.05 h=0.05 i=spline line=1;
symbol5 c=gray v=dot w=0.05 h=0.05 i=spline line=1;
%end;

%if &m = 2 %then %do;
symbol2 c=green v=dot w=0.05 h=0.05 i=spline line=1;
symbol3 c=red v=dot w=0.4 h=0.4 i=spline line=1;
symbol4 c=blue v=dot w=0.05 h=0.05 i=spline line=1;
symbol5 c=gray v=dot w=0.05 h=0.05 i=spline line=1;
%end;

%if &m = 3 %then %do; 
symbol2 c=blue v=dot w=0.05 h=0.05 i=spline line=1;
symbol3 c=green v=dot w=0.05 h=0.05 i=spline line=1;
symbol4 c=red v=dot w=0.2 h=0.2 i=spline line=1;
symbol5 c=gray v=dot w=0.05 h=0.05 i=spline line=1;
%end;

%if &m = 4 %then %do;
symbol2 c=gray v=dot w=0.05 h=0.05 i=spline line=1;
symbol3 c=green v=dot w=0.05 h=0.05 i=spline line=1;
symbol4 c=blue v=dot w=0.05 h=0.05 i=spline line=1;
symbol5 c=red v=dot w=0.2 h=0.2 i=spline line=1;
%end;

%mend;

%macro plot2;
%if &bm1=0 %then %do; ssDNAp1*cot %end; 
%if &bm2=0 %then %do; ssDNAp2*cot %end;
%if &bm3=0 %then %do; ssDNAp3*cot %end;
%if &bm4=0 %then %do; ssDNAp4*cot %end;
%mend;

/* Create another graph showing all 4 models */
goptions reset=all device=gif gsfname=gout;
filename gout "C:\CotQuestU\ReportFiles\FourCotGraphs.gif";  
proc gplot data=cotdata2;

axis1 logbase=10 logstyle=expand; 
axis2 minor=none;
symbol1 v=dot c=black i=none w=0.2 h=0.2;
%sym(&m)

plot ssDNA*cot
/*
	 ssDNAp1*cot
	 ssDNAp2*cot
	 ssDNAp3*cot
	 ssDNAp4*cot
*/
%plot2
	 /haxis=axis1 vaxis=axis2 legend=legend1 overlay;

%let m = %sysfunc(compress(&m));
legend1 label=("Model &m has the lowest AICc.") value=(tick=1 'ssDNA');

run;


/* Create html GIF file pages */
data _null_;
file "C:\CotQuestU\ReportFiles\ResPlots1.htm";
if &bm1 = 0 then
put '<html><head><title>Model 1 Residuals</title></head><body><h3>Model 1 Residual Plots</h3>' /
	'<br><img src=m1_qqPlot.gif><hr><img src=m1_Res_ssDNAp.gif><hr>' /
	'<img src=m1_histogram.gif><hr><img src=m1_Res_Obs.gif></body></html>';
else
put '<html><body>One or more parameters of this model are equal to 0. The model is not valid.</body></html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\ResPlots2.htm";
if &bm2 = 0 then
put '<html><head><title>Model 2 Residuals</title></head><body><h3>Model 2 Residual Plots</h3>' /
	'<br><img src=m2_qqPlot.gif><hr><img src=m2_Res_ssDNAp.gif><hr>' /
	'<img src=m2_histogram.gif><hr><img src=m2_Res_Obs.gif></body></html>';
else
put '<html><body>One or more parameters of this model are equal to 0. The model is not valid.</body></html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\ResPlots3.htm";
if &bm3 = 0 then
put '<html><head><title>Model 3 Residuals</title></head><body><h3>Model 3 Residual Plots</h3>' /
	'<br><img src=m3_qqPlot.gif><hr><img src=m3_Res_ssDNAp.gif><hr>' /
	'<img src=m3_histogram.gif><hr><img src=m3_Res_Obs.gif></body></html>';
else
put '<html><body>One or more parameters of this model are equal to 0. The model is not valid.</body></html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\ResPlots4.htm";
if &bm4 = 0 then
put '<html><head><title>Model 4 Residuals</title></head><body><h3>Model 4 Residual Plots</h3>' /
	'<br><img src=m4_qqPlot.gif><hr><img src=m4_Res_ssDNAp.gif><hr>' /
	'<img src=m4_histogram.gif><hr><img src=m4_Res_Obs.gif></body></html>';
else
put '<html><body>One or more parameters of this model are equal to 0. The model is not valid.</body></html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\CotGraphs.htm";
put '<html><head><title>Cot Graphs</title></head><body><h3>Cot Graphs (m = 1, 2, 3, 4)</h3>' /
	'<br><img src=CotGraph1.gif><hr><br><img src=CotGraph2.gif><hr><img src=CotGraph3.gif><hr><img src=CotGraph4.gif><hr>' /
	'<br><h3>Cot Graphs of All Models</h3><br>' /
	'<img src=FourCotGraphs.gif></body></html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\ResPlots1Fix.htm";
put '<html>Not applicable</html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\ResPlots2Fix.htm";
put '<html>Not applicable</html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\ResPlots3Fix.htm";
put '<html>Not applicable</html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\ResPlots4Fix.htm";
put '<html>Not applicable</html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\Res1Fix.htm";
put '<html>Not applicable</html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\Res2Fix.htm";
put '<html>Not applicable</html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\Res3Fix.htm";
put '<html>Not applicable</html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\Res4Fix.htm";
put '<html>Not applicable</html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\CotGraphsFix.htm";
put '<html>Not applicable</html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\ParametersFix.htm";
put '<html>Not applicable</html>';
run;

data _null_;
file "C:\CotQuestU\ReportFiles\StatisticsFix.htm";
put '<html>Not applicable</html>';
run;


%macro fixZeroDivErrors;

%if &bm2 = 2 %then %do;
%let k_hr2 = 1;
%let k_sl2 = 1;
%end;

%if &bm3 = 3 %then %do;
%let k_hr3 = 1;
%let k_mr3 = 1;
%let k_sl3 = 1;
%end;

%if &bm4 = 4 %then %do;
%let k_vhr4 = 1;
%let k_hr4 = 1;
%let k_mr4 = 1;
%let k_sl4 = 1;
%end;


%mend;

%macro calculateFracOverlaps;


%if %sysevalf(10/&k_hr2 > 0.1/&k_sl2) %then %do;
	%let hr_sl2 = (%sysfunc(log10(10/&k_hr2)) - %sysfunc(log10(0.1/&k_sl2)))/2;
%end;
%else %do;
	%let hr_sl2 = 0;
%end;



%if %sysevalf(10/&k_hr3 > 0.1/&k_mr3) %then %do;
	%let hr_mr3 = (%sysfunc(log10(10/&k_hr3)) - %sysfunc(log10(0.1/&k_mr3)))/2;
%end;
%else %do;
	%let hr_mr3 = 0;
%end;

%if %sysevalf(10/&k_mr3 > 0.1/&k_sl3) %then %do;
	%let mr_sl3 = (%sysfunc(log10(10/&k_mr3)) - %sysfunc(log10(0.1/&k_sl3)))/2;
%end;
%else %do;
	%let mr_sl3 = 0;
%end;



%if %sysevalf(10/&k_vhr4 > 0.1/&k_hr4) %then %do;
	%let vhr_hr4 = (%sysfunc(log10(10/&k_vhr4)) - %sysfunc(log10(0.1/&k_hr4)))/2;
%end;
%else %do;
	%let vhr_hr4 = 0;
%end;

%if %sysevalf(10/&k_hr4 > 0.1/&k_mr4) %then %do;
	%let hr_mr4 = (%sysfunc(log10(10/&k_hr4)) - %sysfunc(log10(0.1/&k_mr4)))/2;
%end;
%else %do;
	%let hr_mr4 = 0;
%end;

%if %sysevalf(10/&k_mr4 > 0.1/&k_sl4) %then %do;
	%let mr_sl4 = (%sysfunc(log10(10/&k_mr4)) - %sysfunc(log10(0.1/&k_sl4)))/2;
%end;
%else %do;
	%let mr_sl4 = 0;
%end;


%mend;


%macro formatter(num);
%if %sysevalf(&num) > %sysevalf(&permis_fraction_overlap) %then %do;
/* anything between do and end is just outputed as a return value of this macro */
<font color=red>%sysfunc(round(%sysevalf(&num*100), 0.01))%</font>
%end;
%else %do;
%sysfunc(round(%sysevalf(&num*100), 0.01))%
%end;
%mend;


data _null_;
/*initialize overlap vars */
%let hr_sl2 = 0;
%let f_hr_sl2 = 0;
%let hr_mr3 = 0;
%let mr_sl3 = 0;
%let f_hr_mr3 = 0;
%let f_mr_sl3 = 0;
%let vhr_hr4 = 0;
%let hr_mr4 = 0;
%let mr_sl4 = 0;
%let f_vhr_hr4 = 0;
%let f_hr_mr4 = 0;
%let f_mr_sl4 = 0;
%let today = %sysfunc(today(), MMDDYY10. ) ;

%fixZeroDivErrors

%calculateFracOverlaps


file "C:\CotQuestU\Report.htm";
put "
<html><head><title>Summary of Cot Analysis</title> 
<style type=text/css>
.container{font-family:verdana,arial,helvetica,sans-serif;}
.container a:link {text-decoration:underline; color:#1E90FF;}
";

put "
.container a:visited {text-decoration:underline; color:#1E90FF;}
.container a:active {text-decoration:underline; color:#1E90FF;}
.container a:hover {text-decoration:none; color:#DC143C;}
";

put "
td.hac{text-align:center; bgcolor:#e6e6e6; padding:5px;}
td.hal{text-align:left; bgcolor:#e6e6e6; padding:5px;}
td.haj{text-align:justify; bgcolor:#e6e6e6;}
";

put "
td.nf1{background-color:#ffcc99; text-align:center; padding:5px;}
td.f1{background-color:#ffccaf; text-align:center; padding:5px;}
";

put "
td.nf2{background-color:#ffff99; text-align:center; padding:5px;}
td.f2{background-color:#ffffcc; text-align:center; padding:5px;}
";

put "
td.nf3{background-color:#99ff99; text-align:center; padding:5px;}
td.f3{background-color:#99ffcc; text-align:center; padding:5px;}
";

put "
td.nf4{background-color:#99ffff; text-align:center; padding:5px;}
td.f4{background-color:#ccffff; text-align:center; padding:5px;}
.zz{font-size:8pt;}
;"

put "
p.title{font-family:monospace; font-size:50pt; color:#DC143C; margin-left:15px;}
p.sub{font-family:verdana,arial,helvetica,sans-serif; font-size:12pt; color:#1E90FF; margin-left:30px;}
</style></head>
";

put "
<body>
";

put "
<div class=container><table class=zz>
<tr><td colspan=8><img alt=CotQuest src='C:\CotQuestG\SAS Scripts\CQ.gif' width=510 height=162></td></tr>
";

put "
<tr><td class=haj colspan=4>Species/Sample: &organism</td><td class=haj colspan=4>Date: &today</td></tr>
";

put "
<tr><td class=haj colspan=4>Algorithm: &alg</td><td class=haj colspan=4>Number of Cot points: &n_obs</td></tr>
<tr><td class=haj colspan=8>Input file: &file</td></tr>
<tr></tr>
<tr></tr>
";

put "
<tr bgcolor=#e6e6e6><td class=hac>&nbsp;</td><td class=hac>AICc,<br>A-sq,<br>W-sq,<br>Convergence</td><td class=hac>Component</td>
";

put "
<td class=hac>Fraction</td><td class=hac>Kinetic<br>Complexity</td><td class=hac><i>k</i></td><td class=hac>Cot&#189;</td><td class=hac>Residual<br>Analysis</td></tr>
";

put "
<tr></tr>
";

put "
<tr><td class=hac rowspan=2 bgcolor=#e6e6e6>Model 1<br><a href='ReportFiles\CotGraph1.gif'>Cot curve</td>
";

put "
<td class=nf1 rowspan=2>&AICc1,<br>&Asq1,<br>&Wsq1,<br>&StatusConv1<br><a href='ReportFiles\NLIN1.htm'>NLIN output</a></td>
";

put "
<td bgcolor=#e6e6e6>Reassociated</td><td class=nf1>&f11</td><td class=nf1>%sysfunc(round(%sysevalf((1015000*&f11)/&k11), 1))</td><td class=nf1>&k11</td><td class=nf1>%sysfunc(round(%sysevalf(1/&k11), 0.0001))</td>
";

put "
<td class=nf1 rowspan=2><a href='ReportFiles\ResPlots1.htm'>Plots</a>&nbsp;|&nbsp;<a href='ReportFiles\Res1.htm'>Data</a></td></tr>
";

put "
<td bgcolor=#e6e6e6>Unreassociated</td><td class=nf1>&f01</td><td class=nf1>&nbsp;</td><td class=nf1>&nbsp;</td><td class=nf1>&nbsp;</td></tr>
";

put "
<tr></tr>";

if &bm2 = 0 then put " 
<tr><td class=hac rowspan=4 bgcolor=#e6e6e6>Model 2<br><a href='ReportFiles\CotGraph2.gif'>Cot curve</td>
";

if &bm2 = 0 then put "
<td class=nf2 rowspan=4>&AICc2,<br>&Asq2,<br>&Wsq2,<br>&StatusConv2<br><a href='ReportFiles\NLIN2.htm'>NLIN output</a></td>
";

if &bm2 = 0 then put " 
<td bgcolor=#e6e6e6>Highly Repetitive</td><td class=nf2>&f_hr2</td><td class=nf2>%sysfunc(round(%sysevalf((1015000*&f_hr2)/&k_hr2),1))</td><td class=nf2>&k_hr2</td><td class=nf2>%sysfunc(round(%sysevalf(1/&k_hr2), 0.0001))</td>
";

if &bm2 = 0 then put "
<td class=nf2 rowspan=4><a href='ReportFiles\ResPlots2.htm'>Plots</a>&nbsp;|&nbsp;<a href='ReportFiles\Res2.htm'>Data</a></td></tr>
";

if &bm2 = 0 then put " 
<tr><td bgcolor=#e6e6e6>Single\Low</td><td class=nf2>&f_sl2</td><td class=nf2>%sysfunc(round(%sysevalf((1015000*&f_sl2)/&k_sl2), 1))</td><td class=nf2>&k_sl2</td><td class=nf2>%sysfunc(round(%sysevalf(1/&k_sl2), 0.0001))</td></tr>
";

if &bm2 = 0 then put " 
<tr><td bgcolor=#e6e6e6>Unreassociated</td><td class=nf2>&f02</td><td class=nf2>&nbsp;</td><td class=nf2>&nbsp;</td><td class=nf2>&nbsp;</td></tr>
";

if &bm2 = 0 then put " 
<tr><td class=hac colspan=5 bgcolor=#ffff99>Fraction overlap. HR-SL: %formatter(&hr_sl2)</td></tr>
";

else put "
<tr><td class=hac bgcolor=#e6e6e6>Model 2</td><td class=nf2 colspan=7>The model is invalid</td></tr>
";

put "
<tr></tr>";

if &bm3 = 0 then put " 
<tr><td class=hac rowspan=5 bgcolor=#e6e6e6>Model 3<br><a href='ReportFiles\CotGraph3.gif'>Cot curve</td>
";

if &bm3 = 0 then put " 
<td class=nf3 rowspan=5>&AICc3,<br>&Asq3,<br>&Wsq3,<br>&StatusConv3<br><a href='ReportFiles\NLIN3.htm'>NLIN output</a></td>
";

if &bm3 = 0 then put " 
<td bgcolor=#e6e6e6>Highly Repetitive</td><td class=nf3>&f_hr3</td><td class=nf3>%sysfunc(round(%sysevalf((1015000*&f_hr3)/&k_hr3), 1))</td><td class=nf3>&k_hr3</td><td class=nf3>%sysfunc(round(%sysevalf(1/&k_hr3), 0.0001))</td>
";

if &bm3 = 0 then put "
<td class=nf3 rowspan=5><a href='ReportFiles\ResPlots3.htm'>Plots</a>&nbsp;|&nbsp;<a href='ReportFiles\Res3.htm'>Data</a></td></tr>
";

if &bm3 = 0 then put " 
<tr><td bgcolor=#e6e6e6>Moderately Repetitive</td><td class=nf3>&f_mr3</td><td class=nf3>%sysfunc(round(%sysevalf((1015000*&f_mr3)/&k_mr3), 1))</td><td class=nf3>&k_mr3</td><td class=nf3>%sysfunc(round(%sysevalf(1/&k_mr3), 0.0001))</td></tr>
";

if &bm3 = 0 then put " 
<tr><td bgcolor=#e6e6e6>Single\Low</td><td class=nf3>&f_sl3</td><td class=nf3>%sysfunc(round(%sysevalf((1015000*&f_sl3)/&k_sl3), 1))</td><td class=nf3>&k_sl3</td><td class=nf3>%sysfunc(round(%sysevalf(1/&k_sl3), 0.0001))</td></tr>
";

if &bm3 = 0 then put " 
<tr><td bgcolor=#e6e6e6>Unreassociated</td><td class=nf3>&f03</td><td class=nf3>&nbsp;</td><td class=nf3>&nbsp;</td><td class=nf3>&nbsp;</td></tr>
";

if &bm3 = 0 then put " 
<tr><td class=hac colspan=5 bgcolor=#99ff99>Fraction overlaps. HR-MR: %formatter(&hr_mr3), MR-SL: %formatter(&mr_sl3)</td></tr>
";

else put "
<tr><td class=hac bgcolor=#e6e6e6>Model 3</td><td class=nf3 colspan=7>The model is invalid</td></tr>
";

put "
<tr></tr>";

if &bm4 = 0 then put " 
<tr><td class=hac rowspan=6 bgcolor=#e6e6e6>Model 4<br><a href='ReportFiles\CotGraph4.gif'>Cot curve</td>
";

if &bm4 = 0 then put "
<td class=nf4 rowspan=6>&AICc4,<br>&Asq4,<br>&Wsq4,<br>&StatusConv4<br><a href='ReportFiles\NLIN4.htm'>NLIN output</a></td>
";

if &bm4 = 0 then put " 
<td bgcolor=#e6e6e6>Very Highly Repetitive</td><td class=nf4>&f_vhr4</td><td class=nf4>%sysfunc(round(%sysevalf((1015000*&f_vhr4)/&k_vhr4), 1))</td><td class=nf4>&k_vhr4</td><td class=nf4>%sysfunc(round(%sysevalf(1/&k_vhr4), 0.0001))</td>
";

if &bm4 = 0 then put "
<td class=nf4 rowspan=6><a href='ReportFiles\ResPlots4.htm'>Plots</a>&nbsp;|&nbsp;<a href='ReportFiles\Res4.htm'>Data</a></td></tr>
";

if &bm4 = 0 then put " 
<tr><td bgcolor=#e6e6e6>Highly Repetitive</td><td class=nf4>&f_hr4</td><td class=nf4>%sysfunc(round(%sysevalf((1015000*&f_hr4)/&k_hr4), 1))</td><td class=nf4>&k_hr4</td><td class=nf4>%sysfunc(round(%sysevalf(1/&k_hr4), 0.0001))</td></tr>
";

if &bm4 = 0 then put " 
<tr><td bgcolor=#e6e6e6>Moderately Repetitive</td><td class=nf4>&f_mr4</td><td class=nf4>%sysfunc(round(%sysevalf((1015000*&f_mr4)/&k_mr4), 1))</td><td class=nf4>&k_mr4</td><td class=nf4>%sysfunc(round(%sysevalf(1/&k_mr4), 0.0001))</td></tr>
";

if &bm4 = 0 then put " 
<tr><td bgcolor=#e6e6e6>Single\Low</td><td class=nf4>&f_sl4</td><td class=nf4>%sysfunc(round(%sysevalf((1015000*&f_sl4)/&k_sl4), 1))</td><td class=nf4>&k_sl4</td><td class=nf4>%sysfunc(round(%sysevalf(1/&k_sl4), 0.0001))</td></tr>
";

if &bm4 = 0 then put " 
<tr><td bgcolor=#e6e6e6>Unreassociated</td><td class=nf4>&f04</td><td class=nf4>&nbsp;</td><td class=nf4>&nbsp;</td><td class=nf4>&nbsp;</td></tr>
";

if &bm4 = 0 then put " 
<tr><td class=hac colspan=5 bgcolor=#99ffff>Fraction overlaps. VHR-HR: %formatter(&vhr_hr4), HR-MR: %formatter(&hr_mr4), MR-SL: %formatter(&mr_sl4)</td></tr>
";

else put "
<tr><td class=hac bgcolor=#e6e6e6>Model 4</td><td class=nf4 colspan=7>The model is invalid</td></tr>
";


put "
<tr></tr><tr></tr><tr></tr><tr></tr>
";

put "
<tr><td class=haj colspan=8>Other Data/Graphs:</td></tr>
";

put "
<tr><td class=haj colspan=8>
<a href='ReportFiles\CotGraphs.htm'>Cot curves for Models 1-4</a><td></tr>
";

put "
<tr><td class=haj colspan=8>
<a href='ReportFiles\GraphPoints.htm'>Cot Points</a><td></tr>
";

put "
<tr><td class=haj colspan=8>
<a href='ReportFiles\Parameters.htm'>Parameter Estimates for Models 1-4</a><td></tr>
";

put "
<tr><td class=haj colspan=8>
<a href='ReportFiles\Statistics.htm'>Stats for Models 1-4</a></td></tr>
";

put '</tr></table></div>';

put '</body></html>';


run;
