/********** SET INPUT FILE, ORGANISM, FITTING ALGORITHM, AND COT GENERATION CONSTANT *********/

/* Input data file */
data cotdata;

/* Input data file */
  %let file = C:\CotQuestU\Sample Data\ecoli.cot;

  infile "'&file'";
  input ssDNA Cot;

/* Enter Species/Sample (it will be included in the output report) */
%let organism = Escherichia coli;

/* Select fitting algorithm. 
Options: 
gauss (recommended) 
marquardt (recommended when parameter correlation is high, > 0.8 or < -0.8)
newton (not recommneded)*/
%let alg = gauss;

/* If Cot points were generated using hydroxyapatite (HAP) chromatography then set this variable to 1
   If S1 nuclease digestion was used then set it to 0.44   */
%let cgc = 1;

/* Set the enforceCap value to 1 if you want the algorithm to enforce the sum of reassociated (f1) 
and non-reassociated (f0) components to stay within 100%. Some datasets won't reach optimal fit if this
option is used. Set it to 0 to disable the cap enforcement. */

%let enforceCap = 1;
/***************************************************************************/


run;

proc sql noprint;
	select count(*) into :n_obs from Cotdata;
quit;

/* run model m=1 with default starting values */
proc nlin data=cotdata method=&alg best=1;      
	bounds f0 f1 k1 >= 0;
	bounds f0 < 1;
	model ssDNA = f0+f1*(1+k1*cot)**(-1*&cgc);
	ODS output parameterestimates=parms1 anova=anova1;
	/* ODS listing parameterestimates=parms1 anova=anova1; */
	OUTPUT out=ssDNAapr1 predicted=ssDNAp;
	parms
		f0=.01 to .26 by .05
		f1=.01 to .96 by .05
		k1=1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2; 
run;

proc sql noprint;
	select Estimate into :f1_orig from Parms1 where Parameter = 'f1';
run;

%macro PlanB;
%if %sysevalf(&f1_orig >= 1) and %sysevalf(&enforceCap = 1) %then 
%do;
proc nlin data=cotdata method=&alg best=1;      
      bounds f0 g k1 >= 0;
      bounds f0 g < 1;
      model ssDNA = f0+(g-f0)*((1+k1*cot)**(-1*&cgc));
      ODS output parameterestimates=parms1 anova=anova1;
      OUTPUT out=ssDNAapr1 predicted=ssDNAp;
      parms
            g=.01 to 1 by .05
            f0=.01 to .26 by .05
            k1=1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2; 
run;

proc sql noprint;
	select Estimate into :gm from Parms1 where Parameter = 'g';
	select Estimate into :f0m from Parms1 where Parameter = 'f0';
	%let f1m = &gm - &f0m;
	insert into Parms1 set Parameter = 'f1', Estimate = &f1m;
quit;

%let enforced = 1;

%end;
%mend;

%let enforced = 0;
/* redo estimates if f1=1*/
%planB



/* create bounds */
data parms1;
	set parms1;
	lowercl = max(0,lowercl);
	uppercl = min(uppercl,1);
	
	run;

/* add residuals */
data ssDNAapr1;
	set ssDNAapr1;
	residual = ssDNA - ssDNAp;
	observation = _N_;
	run;

goptions reset=all device=gif gsfname=gout;
title 'Normal Probability Plot of the Residuals (m=1)';
filename gout 'c:\CotQuestU\OneComponentReportFiles\m1_qqPlot.gif'; 
proc univariate data=ssDNAapr1 normal noprint;	
	var residual;
 	qqplot residual / normal(mu=est sigma=est) pctlscale;
run;

goptions reset=all device=gif gsfname=gout;
title 'Histogram of the Residuals (m=1)';
filename gout 'c:\CotQuestU\OneComponentReportFiles\m1_histogram.gif'; 
proc univariate data=ssDNAapr1 normal noprint;
	var residual;
 	histogram residual / cfill=ltgray normal(color=red MU=0) kernel(color=blue); /*midpoints= -0.03 to 0.03 by 0.001*/
	inset normal(ad adpval cvm cvmpval) / cfill  = ywh format = 7.6;
	ODS output GoodnessOfFit=univ1;
run;

goptions reset=all device=gif gsfname=gout;
title 'Residuals Versus the Fitted Values (m=1)';
filename gout 'c:\CotQuestU\OneComponentReportFiles\m1_Res_ssDNAp.gif'; 
proc gplot data=ssDNAapr1;
	plot residual*ssDNAp /vref=0;	
	symbol1 v=dot c=blue i=none w=0.5 h=0.5;
run;

goptions reset=all device=gif gsfname=gout;
title 'Residuals Versus the Order of the Data (m=1)';
filename gout 'c:\CotQuestU\OneComponentReportFiles\m1_Res_Obs.gif'; 
proc gplot data=ssDNAapr1;
	plot residual*observation /vref=0;
	symbol1 v=dot c=red ci=blue i=join w=0.5 h=0.5;
run;
title;


/* create model fit comparison output dataset including AIC */
data anova1;
	set anova1;
	model = 1;
	if source = 'Error' then do;
		AIC = &n_obs*log(SS/&n_obs)+2*(&n_obs-DF+1);
		AICc = AIC + 2*(&n_obs-DF+1)*((&n_obs-DF+1)+1)/(&n_obs-(&n_obs-DF+1)-1);
		end;
	run;



/* print summary results */
ods html path = 'c:\CotQuestU\OneComponentReportFiles' (url=none) file='Parameters.htm'; 
proc print data=parms1;
run;
ods html close;


ods html file='c:\CotQuestU\OneComponentReportFiles\Statistics.htm';
proc print data=anova1;
run;
ods html close;


ods html file='c:\CotQuestU\OneComponentReportFiles\Res1.htm';
proc print data=ssDNAapr1;  
run;
ods html close;


%macro bm;
	%if %sysevalf(&f01<=0) or %sysevalf(&f11<=0) or %sysevalf(&k11<=0) %then %let bm1 = 1;
	
%mend;

proc sql noprint;

	%let bm1 = 0;
	
	select Estimate into :f01 from parms1 where Parameter = 'f0';
	select Estimate into :f11 from parms1 where Parameter = 'f1';
	select Estimate into :k11 from parms1 where Parameter = 'k1';
	
	%bm

	select max(Cot) into :ubn from Cotdata;
	select min(Cot) into :lbn from Cotdata;
	
	
	select AICc into :AICc from anova1 where Source = 'Error';
	select Stat into :Asq from univ1 where  TestLab = 'A-Sq'; 
	select Stat into :Wsq from univ1 where  TestLab = 'W-Sq'; 
	

quit;



%macro equations;

%if &bm1 = 0 %then %do; 
	ssDNAp1 = &f01+&f11*(1+&k11*cot)**(-1*&cgc); 
%end; 


%mend;

/*Calculate 3001 points for a high-precision graph extended half order of magnitude on both ends */
data cotscan; 

  do iterations = log10(&lbn/5) to log10(&ubn*5) by (log10(&ubn*5)-log10(&lbn/5))/3000; 
	cot = 10**(iterations);
	
	%equations
	output;  
  end; 

  drop iterations;
run;

/* Merge with the existing dataset */
data cotdata2; 
  set cotdata cotscan; 
run;

proc sort data=cotdata2;
   by cot;
run;

ods html file='c:\CotQuestU\OneComponentReportFiles\GraphPoints.htm';
proc print data=cotdata2;
format _all_ 7.6;
run; 
ods html close;

/* Model graph */

%macro mplot;
%if &bm1=0 %then %do;
/* title 'Cot Curve'; */
%let k111 = %sysevalf(0.1/&k11);
%let k112 = %sysevalf(1/&k11);
%let k113 = %sysevalf(10/&k11);


plot ssDNA*Cot ssDNAp1*Cot 
	/haxis=axis1 vaxis=axis2
	href=(&k111 &k112 &k113) 
	chref=(blue blue blue) lhref=(46 1 46)
	overlay;
%end;
%else %do;
title 'Model is invalid, at least one of its parameters is equal to 0.';
plot ssDNA*Cot /haxis=axis1 vaxis=axis2;
%end;

%mend;
goptions reset=all device=gif gsfname=gout;
filename gout 'c:\CotQuestU\OneComponentReportFiles\CotGraph.gif'; 

proc gplot data=cotdata2;

axis1 logbase=10 logstyle=expand; 
axis2 minor=none;
%mplot

symbol1 v=dot c=black i=none w=0.2 h=0.2;
symbol2 c=red v=dot w=0.1 h=0.1 i=spline line=1;

run;


data _null_;
file 'c:\CotQuestU\OneComponentReportFiles\ResPlots1.htm';
if &bm1 = 0 then
put '<html><head><title>Model 1 Residuals</title></head><body><h3>Model 1 Residual Plots</h3>' /
	'<br><img src=m1_qqPlot.gif><hr><img src=m1_Res_ssDNAp.gif><hr>' /
	'<img src=m1_histogram.gif><hr><img src=m1_Res_Obs.gif></body></html>';
else
put '<html><body>One or more pameters of this model are equal to 0. The model is not valid.</body></html>';
run;


data _null_;
file 'c:\CotQuestU\OneComponentReport.htm';

put "<html><head><style type=text/css>h3{page-break-after:auto; font-size:15.0pt;	font-family:Verdana; color:#4A0022;";
put "font-weight:normal; text-transform:uppercase; text-align:center;} a:link, span.MsoHyperlink{font-family:Verdana;";
put "color:#4A0022; text-decoration:underline; font-size:11.0pt; font-weight:normal; text-transform:uppercase;}";
put "a:visited, span.MsoHyperlinkFollowed{font-family:Verdana; color:#4A0022; text-decoration:underline; font-size:11.0pt;";
put "font-weight:normal; text-transform:uppercase;} a:hover{background-color:#4A0022; color:White;}";
put "td.hdr{background-color:#4A0022; color:White; font-family:Verdana;	font-size:11.0pt;	font-weight:600; text-transform:uppercase;}";
put "</style></head><body><h3>CotQuest Report</h3><iframe src='OneComponentReportFiles/Summary.htm' frameborder=no scrolling=no height=200  width=100%></iframe>";
put "<img src=OneComponentReportFiles/CotGraph.gif /><table cellpadding=5 cellspacing=10 width=100%><tr>";
put "<td><a href=OneComponentReportFiles/Parameters.htm>Parameter Estimates</a></td><td><a href=OneComponentReportFiles/Statistics.htm>Statistics</a></td>";
put "<td colspan=2><a href=OneComponentReportFiles/GraphPoints.htm>Cot Graph Points</a></td>";
put "<td><a href=OneComponentReportFiles/ResPlots1.htm>Residual Plots</a></td><td><a href=OneComponentReportFiles/Res1.htm>Residuals</a></td>";
put "</tr></table></body></html>
";
run;

data _null_;
file "c:\CotQuestU\OneComponentReportFiles\Summary.htm";
%let today = %sysfunc(today(), MMDDYY10. ) ;

put "
<html><table width=100%><tr><td valign=top>Date: &today <br>Algorithm: &alg 
";
put "<br><b><font color=navy>Fitting Statistics: </font></b><br>AICc: &AICc <br> A-sq: &Asq <br> W-sq: &Wsq</td>";

put "
<td>Input File: &file <br>Species/Sample: &organism 
";
put "
<br><b><font color=navy>Cot Analysis: </font></b><br>Reassociated Fraction: &f11, Non-reassociated Fraction: &f01 
<br> Kinetic Complexity: %sysevalf((1015000*&f11)/&k11) 
<br> K: &k11
<br> Cot1/2: %sysfunc(round(%sysevalf(1/&k11), 0.0001))</td></tr>
<tr><td colspan=2>";

if %sysevalf(%sysevalf(&f11 + &f01) > 1) then
put "<font color=red>Warning: Sum of components exceeds 100%.</font>";

if &enforced = 1 then
put "<font color=red>Warning: Sum of components was forced to stay within 100%. Evaluation terminated before optimal fit was reached.</font>";

put '</td></tr></table></html>';


run;



